<?php

namespace Lession\Lession\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Lession\Lession\Http\Requests\LessionRequest;
use Lession\Lession\Interfaces\LessionRepositoryInterface;
use Lession\Lession\Models\Lession;

/**
 * Resource controller class for lession.
 */
class LessionResourceController extends BaseController
{

    /**
     * Initialize lession resource controller.
     *
     * @param type LessionRepositoryInterface $lession
     *
     * @return null
     */
    public function __construct(LessionRepositoryInterface $lession)
    {
        parent::__construct();
        $this->repository = $lession;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Lession\Lession\Repositories\Criteria\LessionResourceCriteria::class);
    }

    /**
     * Display a list of lession.
     *
     * @return Response
     */
    public function index(LessionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Lession\Lession\Repositories\Presenter\LessionPresenter::class)
                ->$function();
        }

        $lessions = $this->repository->paginate();

        return $this->response->title(trans('lession::lession.names'))
            ->view('lession::lession.index', true)
            ->data(compact('lessions', 'view'))
            ->output();
    }

    /**
     * Display lession.
     *
     * @param Request $request
     * @param Model   $lession
     *
     * @return Response
     */
    public function show(LessionRequest $request, Lession $lession)
    {

        if ($lession->exists) {
            $view = 'lession::lession.show';
        } else {
            $view = 'lession::lession.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('lession::lession.name'))
            ->data(compact('lession'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new lession.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LessionRequest $request)
    {

        $lession = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('lession::lession.name')) 
            ->view('lession::lession.create', true) 
            ->data(compact('lession'))
            ->output();
    }

    /**
     * Create new lession.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LessionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $lession                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('lession::lession.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('lession/lession/' . $lession->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/lession/lession'))
                ->redirect();
        }

    }

    /**
     * Show lession for editing.
     *
     * @param Request $request
     * @param Model   $lession
     *
     * @return Response
     */
    public function edit(LessionRequest $request, Lession $lession)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('lession::lession.name'))
            ->view('lession::lession.edit', true)
            ->data(compact('lession'))
            ->output();
    }

    /**
     * Update the lession.
     *
     * @param Request $request
     * @param Model   $lession
     *
     * @return Response
     */
    public function update(LessionRequest $request, Lession $lession)
    {
        try {
            $attributes = $request->all();

            $lession->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('lession::lession.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('lession/lession/' . $lession->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('lession/lession/' . $lession->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the lession.
     *
     * @param Model   $lession
     *
     * @return Response
     */
    public function destroy(LessionRequest $request, Lession $lession)
    {
        try {

            $lession->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('lession::lession.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('lession/lession/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('lession/lession/' . $lession->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple lession.
     *
     * @param Model   $lession
     *
     * @return Response
     */
    public function delete(LessionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('lession::lession.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('lession/lession'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/lession/lession'))
                ->redirect();
        }

    }

    /**
     * Restore deleted lessions.
     *
     * @param Model   $lession
     *
     * @return Response
     */
    public function restore(LessionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('lession::lession.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/lession/lession'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/lession/lession/'))
                ->redirect();
        }

    }

}
